
from IPython.core.magic import register_cell_magic
from IPython.display import HTML
import re
from saspy.SASLogLexer import SASLogStyle, SASLogLexer
from saspy.sasbase import SASsession
from pygments.formatters import HtmlFormatter
from pygments import highlight
import warnings

def SASMagic(sas, sys='viya') :
    warnings.filterwarnings('ignore',module='saspy')
    
    if sys == 'viya' :
        sas.submit("""
        proc template;
          define style Styles.Hangul;
          parent = Styles.HTMLBlue;
          style graphfonts from graphfonts /
                'NodeDetailFont' = ("Noto Sans KR",7pt)
                'NodeInputLabelFont' = ("Noto Sans KR",9pt)
                'NodeLabelFont' = ("Noto Sans KR",9pt)
                'NodeTitleFont' = ("Noto Sans KR",9pt)
                'GraphDataFont' = ("Noto Sans KR",7pt)
                'GraphUnicodeFont' = ("Noto Sans KR",9pt)
                'GraphValueFont' = ("Noto Sans KR",9pt)
                'GraphLabel2Font' = ("Noto Sans KR",10pt)
                'GraphLabelFont' = ("Noto Sans KR",10pt)
                'GraphFootnoteFont' = ("Noto Sans KR",10pt)
                'GraphTitleFont' = ("Noto Sans KR",11pt,bold)
                'GraphTitle1Font' = ("Noto Sans KR",14pt,bold)
                'GraphAnnoFont' = ("Noto Sans KR",10pt);         
          end;
        run;
        """)
        sas.HTML_Style = "Hangul"
        sas.submit("""
        filename storage FILESRVC folderpath='/Public/'; 
        %include storage(sas7bdat, KOSIS_MACRO_V3_5, ecos, enara, localfinance, POSTDATA, ECOS3D); 
        %LET MARKER=MARKERS MARKERATTRS=(SYMBOL=CIRCLEFILLED SIZE=11); 
        %LET DATALABEL=DATALABEL DATALABELATTRS=(SIZE=11); 
        %LET printit=%str(proc print data=raw(obs=3);run;); 
        %LET xaxis=xaxis type=discrete valueattrs=(size=10.5) labelattrs=(size=10.5) display=(nolabel); 
        %LET yaxis=yaxis grid valueattrs=(size=10.5) labelattrs=(size=10.5) labelpos=top;         
        cas casauto; 
        caslib _all_ assign; 
        options dlcreatedir; 
        options casdatalimit=all;
        caslib c datasource=(srctype="path") path="/home/&sysuserid/casuser" sessref=casauto;
        LIBNAME ASOS './public/';
        ods graphics/imagemap;
        """)
        print("알림 : KOSIS_MACRO_V3_5, ECOS, ECOS3, ENARA, POSTDATA, MODIFY_DATA, LOCALFINANCE, SAS7BDAT 매크로를 로드하였습니다.")
        print("알림 : 매크로 변수 MARKER, DATALABEL, PRINTIT, XAXIS, YAXIS를 로드하였습니다.")
        print("알림 : 모든 CAS 라이브러리를 로드하였습니다. ODS GRAPHICS IMAGEMAP 옵션을 로드하였습니다.")
        print("알림 : Jupyter Notebook Cell Magic %%SASK를 로드하였습니다.")
        
        
    if sys == 'oda' :
        sas.submit("""
        proc template;
          define style Styles.Hangul;
          parent = Styles.HTMLBlue;
          style graphfonts from graphfonts /
                'NodeDetailFont' = ("Gulim",7pt)
                'NodeInputLabelFont' = ("Gulim",9pt)
                'NodeLabelFont' = ("Gulim",9pt)
                'NodeTitleFont' = ("Gulim",9pt)
                'GraphDataFont' = ("Gulim",7pt)
                'GraphUnicodeFont' = ("Gulim",9pt)
                'GraphValueFont' = ("Gulim",9pt)
                'GraphLabel2Font' = ("Gulim",10pt)
                'GraphLabelFont' = ("Gulim",10pt)
                'GraphFootnoteFont' = ("Gulim",10pt)
                'GraphTitleFont' = ("Gulim",11pt,bold)
                'GraphTitle1Font' = ("Gulim",14pt,bold)
                'GraphAnnoFont' = ("Gulim",10pt);         
          end;
        run;
        """)
        sas.HTML_Style = "Hangul"
        sas.submit("""
        options validvarname=any;
        
        %let temp=%sysfunc(getoption(work));
        filename kosis "&temp/MACROS.sas";
        proc http url="https://www.googleapis.com/drive/v3/files/15L96rVErDU10g9o-U-8LnLo9XtvZjWmz?alt=media&key=AIzaSyBfJIzuu9x7AZjgtr0UhbrxNTz0vqbYWv0" method='GET' out=kosis;
        run;
        %include "&temp/MACROS.sas";
        %symdel temp;
                       
        %LET MARKER=MARKERS MARKERATTRS=(SYMBOL=CIRCLEFILLED SIZE=11); 
        %LET DATALABEL=DATALABEL DATALABELATTRS=(SIZE=11); 
        %LET printit=%str(proc print data=raw(obs=3);run;); 
        %LET xaxis=xaxis type=discrete valueattrs=(size=10.5) labelattrs=(size=10.5) display=(nolabel); 
        %LET yaxis=yaxis grid valueattrs=(size=10.5) labelattrs=(size=10.5) labelpos=top;
        ods graphics/imagemap;
        """)
        print("알림 : KOSIS_MACRO_V3_5, ECOS, ECOS3, ENARA, POSTDATA, MODIFY_DATA, LOCALFINANCE, SAS7BDAT 매크로를 로드하였습니다.")
        print("알림 : 매크로 변수 MARKER, DATALABEL, PRINTIT, XAXIS, YAXIS를 로드하였습니다.")
        print("알림 : ODS GRAPHICS IMAGEMAP 옵션을 로드하였습니다.")
        print("알림 : Jupyter Notebook Cell Magic %%SASK를 로드하였습니다.")        
            
    def _which_display(sas, log, output):
      lst_len = 30762
      lines = re.split(r'[\n]\s*', log)
      i = 0
      elog = []
      for line in lines:
          i += 1
          e = []
          if line[sas.logoffset:].startswith('ERROR'):
              e = lines[(max(i - 15, 0)):(min(i + 16, len(lines)))]
          elog = elog + e
      if len(elog) == 0 and len(output) > lst_len:   # no error and LST output
          return HTML(output)
      elif len(elog) == 0 and len(output) <= lst_len:   # no error and no LST
          color_log = highlight(log, SASLogLexer(), HtmlFormatter(full=True, style=SASLogStyle, lineseparator="<br>"))
          return HTML(color_log)
      elif len(elog) > 0 and len(output) <= lst_len:   # error and no LST
          color_log = highlight(log, SASLogLexer(), HtmlFormatter(full=True, style=SASLogStyle, lineseparator="<br>"))
          return HTML(color_log)
      else:   # errors and LST
          color_log = highlight(log, SASLogLexer(), HtmlFormatter(full=True, style=SASLogStyle, lineseparator="<br>"))
          return HTML(color_log + output)

    @register_cell_magic
    def SASK(line, cell):
        sas.submit("proc optsave out=__jupyterSASKernel__; run;")
        if len(line) > 0 :
            res = sas.submit("ods layout gridded columns=" + str(line) + " advance=table;" + cell + "ods layout end;")
        else :
            res = sas.submit(cell)        
        dis = _which_display(sas, res['LOG'], res['LST'])
        sas.submit("proc optload data=__jupyterSASKernel__; run;")
        return dis

    warnings.filterwarnings('default')


#df(데이터프레임), rv(지역변수, string), dt(분석변수, string)
def p_domap(df, rv, dt, label):
  import geopandas as gpd
  import pandas as pd
  import folium
  import os
  sigungu_geo = 'https://drive.google.com/uc?export=download&id=1lbYoieCkDHljHA6tFFYi2ZRmpFbm_vml'
  df_sigungu_geo = gpd.read_file(sigungu_geo) # 시도/시군구명, geocode, polygon 정보를 담고 있는 geojson 파일을 데이터프레임으로 바꾸어라
  df['SIGUNGU_CD'] = df[rv].apply(str) # sasdataset의 지역변수를 string으로 바꾸어 SAS 데이터프레임의 SIGUNGU_CD 변수로 저장한다
  df_sigungu_merged = df_sigungu_geo.merge(df, on='SIGUNGU_CD') # SAS의 SIGUNGU_CD 데이터셋과 GEOJSON을 데이터프레임으로 변환한 자료를 결합하라(이를 통해 폴리곤 정보가 들어가게 됨)
  df_sigungu_merged['tv']= df_sigungu_merged['SIGUNGU_CD'].str.slice(0, 2) #지역코드(SIGUNGU_CD) 앞 2자리를 자르라
  df_sigungu_merged['nv']= df_sigungu_merged['SIGUNGU_CD'].str.len() #지역코드(SIGUNGU_CD) 문자열의 개수를 반환하라(5 또는 2의 값을 가짐)
  try :
    if df_sigungu_merged.duplicated(['tv', 'year'], keep=False).value_counts()[1] > 0 : # 지역코드 앞 두자리와 연도변수를 종합적으로 고려했을 때 중복의 개수가 0보다 많으면 True, 없으면 KeyError를 반환
      # 데이터셋에서 (중복 관측점 & 지역코드 문자열이 2개를 초과하는 관측점; 이는 시도자료를 제외하고 시군구 자료를 확보하기 위함) OR 중복이 없는 관측점(세종과 제주를 포함하기 위함)
      df_sigungu_merged = df_sigungu_merged[((df_sigungu_merged.duplicated(['tv', 'year'], keep=False)) & (df_sigungu_merged['nv']>2)) | (df_sigungu_merged.duplicated(['tv', 'year'], keep=False)==0)].copy()
  except : # KeyError, IndexError 오류 무시하라(시도 데이터셋인 경우 if문 다음에 바로 여기로 옴)
    pass
  df_sigungu_merged[['SIGUNGU_CD', 'SIGUNGU_NM', 'geometry']].to_file("a.json", driver='GeoJSON') # merge 데이터셋에서 geocode 데이터셋을 a.json으로 저장하라
  m = folium.Map(location=[36, 128], width=800, height=800, tiles="OpenStreetMap", zoom_start=7)
  folium.Choropleth(geo_data="a.json", data=df_sigungu_merged, columns=['SIGUNGU_CD', dt],key_on='feature.properties.SIGUNGU_CD',fill_color='YlOrRd',fill_opacity=0.7,line_opacity=0.5,legend_name=label).add_to(m)
  style_function = lambda x: {'fillColor': '#ffffff', 'color':'#000000', 'fillOpacity': 0.1, 'weight': 0.1}
  highlight_function = lambda x: {'fillColor': '#000000','color':'#000000', 'fillOpacity': 0.50, 'weight': 0.1}
  LABEL = folium.features.GeoJson(df_sigungu_merged, style_function=style_function, control=False, highlight_function=highlight_function, 
                                  tooltip=folium.features.GeoJsonTooltip(fields=['SIGUNGU_NM', 'year', dt],aliases=['지역: ', '연도: ', label],style=("background-color: white; color: #333333; font-family: arial; font-size: 12px; padding: 10px;")))
  m.add_child(LABEL)
  m.keep_in_front(LABEL)
  folium.LayerControl().add_to(m)
  os.remove("a.json") # a.json을 삭제하라
  return m


def convert_code(df):
 import pandas as pd
 pd.set_option('mode.chained_assignment',  None)
 name = pd.read_csv('https://drive.google.com/uc?export=download&id=1dgXL5NVgt6Xb52j3K5ROTnctvir-c7Qs', encoding='utf-8') #지역명이 중복되지 않는 지역 목록
 df_sorted = df.sort_values(["C1", "C1_NM"], ascending = (True, True)) #시도명, 시군구명 오름차순 정렬
 df_name = df_sorted.merge(name, on='C1_NM', how='left') 
 df_name['Counter'] = df_name.groupby('C1_NM').cumcount()+1
 df_sigungu = df_name

 for n in df_sigungu.index: #지역명이 중복되는 지역은 따로 입력
 
  if df_sigungu['C1_NM'][n] == '중구' and df_sigungu['Counter'][n] == 1:
            df_sigungu['FULL_NM'][n] = '서울특별시 중구'
  elif df_sigungu['C1_NM'][n] == '중구' and df_sigungu['Counter'][n] == 2:
            df_sigungu['FULL_NM'][n] = '부산광역시 중구'
  elif df_sigungu['C1_NM'][n] == '중구' and df_sigungu['Counter'][n] == 3:
            df_sigungu['FULL_NM'][n] = '대구광역시 중구'
  elif df_sigungu['C1_NM'][n] == '중구' and df_sigungu['Counter'][n] == 4:
            df_sigungu['FULL_NM'][n] = '인천광역시 중구'
  elif df_sigungu['C1_NM'][n] == '중구' and df_sigungu['Counter'][n] == 5:
            df_sigungu['FULL_NM'][n] = '대전광역시 중구'
  elif df_sigungu['C1_NM'][n] == '중구' and df_sigungu['Counter'][n] == 6:
            df_sigungu['FULL_NM'][n] = '울산광역시 중구'
            
  elif df_sigungu['C1_NM'][n] == '동구' and df_sigungu['Counter'][n] == 1:
            df_sigungu['FULL_NM'][n] = '부산광역시 동구'
  elif df_sigungu['C1_NM'][n] == '동구' and df_sigungu['Counter'][n] == 2:
            df_sigungu['FULL_NM'][n] = '대구광역시 동구'
  elif df_sigungu['C1_NM'][n] == '동구' and df_sigungu['Counter'][n] == 3:
            df_sigungu['FULL_NM'][n] = '인천광역시 동구'
  elif df_sigungu['C1_NM'][n] == '동구' and df_sigungu['Counter'][n] == 4:
            df_sigungu['FULL_NM'][n] = '광주광역시 동구'
  elif df_sigungu['C1_NM'][n] == '동구' and df_sigungu['Counter'][n] == 5:
            df_sigungu['FULL_NM'][n] = '대전광역시 동구'
  elif df_sigungu['C1_NM'][n] == '동구' and df_sigungu['Counter'][n] == 6:
            df_sigungu['FULL_NM'][n] = '울산광역시 동구'

  elif df_sigungu['C1_NM'][n] == '남구' and df_sigungu['Counter'][n] == 1:
            df_sigungu['FULL_NM'][n] = '부산광역시 남구'
  elif df_sigungu['C1_NM'][n] == '남구' and df_sigungu['Counter'][n] == 2:
            df_sigungu['FULL_NM'][n] = '대구광역시 남구'
  elif df_sigungu['C1_NM'][n] == '남구' and df_sigungu['Counter'][n] == 3:
            df_sigungu['FULL_NM'][n] = '광주광역시 남구'
  elif df_sigungu['C1_NM'][n] == '남구' and df_sigungu['Counter'][n] == 4:
            df_sigungu['FULL_NM'][n] = '울산광역시 남구'
  elif df_sigungu['C1_NM'][n] == '남구' and df_sigungu['Counter'][n] == 5:
            df_sigungu['FULL_NM'][n] = '경상북도 포항시 남구'

  elif df_sigungu['C1_NM'][n] == '서구' and df_sigungu['Counter'][n] == 1:
            df_sigungu['FULL_NM'][n] = '부산광역시 서구'
  elif df_sigungu['C1_NM'][n] == '서구' and df_sigungu['Counter'][n] == 2:
            df_sigungu['FULL_NM'][n] = '대구광역시 서구'
  elif df_sigungu['C1_NM'][n] == '서구' and df_sigungu['Counter'][n] == 3:
            df_sigungu['FULL_NM'][n] = '인천광역시 서구'
  elif df_sigungu['C1_NM'][n] == '서구' and df_sigungu['Counter'][n] == 4:
            df_sigungu['FULL_NM'][n] = '광주광역시 서구'
  elif df_sigungu['C1_NM'][n] == '서구' and df_sigungu['Counter'][n] == 5:
            df_sigungu['FULL_NM'][n] = '대전광역시 서구'

  elif df_sigungu['C1_NM'][n] == '북구' and df_sigungu['Counter'][n] == 1:
            df_sigungu['FULL_NM'][n] = '부산광역시 북구'
  elif df_sigungu['C1_NM'][n] == '북구' and df_sigungu['Counter'][n] == 2:
            df_sigungu['FULL_NM'][n] = '대구광역시 북구'
  elif df_sigungu['C1_NM'][n] == '북구' and df_sigungu['Counter'][n] == 3:
            df_sigungu['FULL_NM'][n] = '광주광역시 북구'
  elif df_sigungu['C1_NM'][n] == '북구' and df_sigungu['Counter'][n] == 4:
            df_sigungu['FULL_NM'][n] = '울산광역시 북구'
  elif df_sigungu['C1_NM'][n] == '북구' and df_sigungu['Counter'][n] == 5:
            df_sigungu['FULL_NM'][n] = '경상북도 포항시 북구'            

  elif df_sigungu['C1_NM'][n] == '강서구' and df_sigungu['Counter'][n] == 1:
            df_sigungu['FULL_NM'][n] = '서울특별시 강서구'
  elif df_sigungu['C1_NM'][n] == '강서구' and df_sigungu['Counter'][n] == 2:
            df_sigungu['FULL_NM'][n] = '부산광역시 강서구'

  elif df_sigungu['C1_NM'][n] == '강서구' and df_sigungu['Counter'][n] == 1:
            df_sigungu['FULL_NM'][n] = '서울특별시 강서구'
  elif df_sigungu['C1_NM'][n] == '강서구' and df_sigungu['Counter'][n] == 2:
            df_sigungu['FULL_NM'][n] = '부산광역시 강서구'

  elif df_sigungu['C1_NM'][n] == '고성군' and df_sigungu['Counter'][n] == 1:
            df_sigungu['FULL_NM'][n] = '강원도 고성군'
  elif df_sigungu['C1_NM'][n] == '고성군' and df_sigungu['Counter'][n] == 2:
            df_sigungu['FULL_NM'][n] = '경상남도 고성군'

  else:
            df_sigungu['FULL_NM'][n] = df_sigungu['FULL_NM'][n]
 
 code = pd.read_csv('https://drive.google.com/uc?export=download&id=1Qk77GoEfJsnHMRqQk38O9rfKbuExtHga', encoding='utf-8') #한국행정구역분류(2021.12.31)
 df_sigungu_code = df_sigungu.merge(code, on='FULL_NM', how='left')
 df_sigungu_code.dropna()
 df_sigungu_code['STAT_CODE'] = df_sigungu_code['STAT_CODE'].astype(str).replace('\.0', '', regex=True)
 df_sigungu_code['ADM_CODE'] = df_sigungu_code['ADM_CODE'].astype(str).replace('\.0', '', regex=True)
 df_sigungu_code['LAW_CODE'] = df_sigungu_code['LAW_CODE'].astype(str).replace('\.0', '', regex=True)
 
 df_sigungu_code_index = df_sigungu_code.drop(['Counter'], axis = 1)
 return df_sigungu_code_index


import saspy
import sys
import requests
import warnings

version=str(sys.version_info.major)+'.'+str(sys.version_info.minor)
url = 'https://www.googleapis.com/drive/v3/files/1wQkHbgrcF03hN8CrIFLK4zsqU-ckVRyK?alt=media&key=AIzaSyBfJIzuu9x7AZjgtr0UhbrxNTz0vqbYWv0'
dst = '/usr/local/lib/python'+version+'/dist-packages/saspy/java/iomclient/sas.rutil.jar'
open(dst, 'wb').write(requests.get(url).content)
url = 'https://www.googleapis.com/drive/v3/files/1wUiEDOu2UMsW6394MrC0s4D-FHPAlt8o?alt=media&key=AIzaSyBfJIzuu9x7AZjgtr0UhbrxNTz0vqbYWv0'
dst = '/usr/local/lib/python'+version+'/dist-packages/saspy/java/iomclient/sas.rutil.nls.jar'
open(dst, 'wb').write(requests.get(url).content)
url = 'https://www.googleapis.com/drive/v3/files/1wTOLejKU5UKw61KGu4oT_WM4ZdWOAdqu?alt=media&key=AIzaSyBfJIzuu9x7AZjgtr0UhbrxNTz0vqbYWv0'
dst = '/usr/local/lib/python'+version+'/dist-packages/saspy/java/iomclient/sastpj.rutil.jar'
open(dst, 'wb').write(requests.get(url).content)
    
def SASLogin(id, pw, sys='viya',server="Asia Pacific Home Region 1") :
  if sys == 'viya' :
    sas = saspy.SASsession(ip='147.47.206.193', user=str(id), pw=str(pw), verify=False, context='SAS Studio compute context', encoding='utf-8')
  if (sys == 'oda' and server == "US Home Region 1") :
    sas = saspy.SASsession(java='/usr/bin/java', iomhost = ['odaws01-usw2.oda.sas.com','odaws02-usw2.oda.sas.com','odaws03-usw2.oda.sas.com','odaws04-usw2.oda.sas.com'], iomport=8591, encoding='utf-8', omruser=str(id), omrpw=str(pw))
  if (sys == 'oda' and server == "US Home Region 2") :
    sas = saspy.SASsession(java='/usr/bin/java', iomhost =['odaws01-usw2-2.oda.sas.com','odaws02-usw2-2.oda.sas.com'], iomport=8591, encoding='utf-8', omruser=str(id), omrpw=str(pw))
  if (sys == 'oda' and server == "European Home Region 1") :
    sas = saspy.SASsession(java='/usr/bin/java', iomhost =['odaws01-euw1.oda.sas.com','odaws02-euw1.oda.sas.com'], iomport=8591, encoding='utf-8', omruser=str(id), omrpw=str(pw))
  if (sys == 'oda' and server == "Asia Pacific Home Region 1") :
    sas = saspy.SASsession(java='/usr/bin/java', iomhost =['odaws01-apse1.oda.sas.com','odaws02-apse1.oda.sas.com'], iomport=8591, encoding='utf-8', omruser=str(id), omrpw=str(pw))
  if (sys == 'oda' and server == "Asia Pacific Home Region 2") :
    sas = saspy.SASsession(java='/usr/bin/java', iomhost =['odaws01-apse1-2.oda.sas.com','odaws02-apse1-2.oda.sas.com'], iomport=8591, encoding='utf-8', omruser=str(id), omrpw=str(pw))
    return sas


from setuptools import setup
setup(
	  name='kolabpy', #module 이름
	  version='1.0.3.1',  
	  description='',
	  long_description= 'Contributor:apjh2529@naver.com', 
	  author='Kolab',
	  author_email='kkd8326@snu.ac.kr',
	  url='https://aric.snu.ac.kr/',
	  license='MIT',
	  py_modules=['pymodule'], #업로드할 module
	  python_requires='>=3', #파이썬 버전 
	  install_requires=['saspy==5.3.0', 'geopandas>=0.10', 'folium>=0.8'], #module 필요한 다른 module
	  packages=['kolabpy'] #업로드할 module이 있는 폴더
)